/******************************************************************************
 *   @file   adf4157.c
 *   @brief  Implementation of ADF4157 Driver for MicroBlaze processor.
 *   @author ATofan (alexandru.tofan@analog.com)
 *
 *******************************************************************************
 * Copyright 2011(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *   SVN Revision: 788
 ******************************************************************************/
#include "../drivers/gpio.h"
#include "../drivers/spi.h"
#include "ADF4157.h"

#define ADF4157_GLOBALS

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
char txBuffer[3] = {0,}; // TX SPI Buffer
char rxBuffer[3] = {0,}; // RX SPI Buffer

char adf4157ModR0Prev = 0;
char adf4157ModR1Prev = 0;
char adf4157ModR2Prev = 0;
char adf4157ModR3Prev = 0;
char adf4157ModR4Prev = 0;
char adf4157ModAllPrev = 0;

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
// R0 + R1
volatile char* 		   	adf4157MuxOut		= &ADF4157_Data[0].chData;
volatile int*  		   	adf4157IntVal		= &ADF4157_Data[1].iData;
volatile unsigned int* 	adf4157FracVal		= &ADF4157_Data[2].uiData;

// R2
volatile char* 		   	adf4157CsrEn		= &ADF4157_Data[3].chData;
volatile char*		   	adf4157CurrSet	 	= &ADF4157_Data[4].chData;
volatile char* 		   	adf4157Prescaler	= &ADF4157_Data[5].chData;
volatile char*		   	adf4157RDiv2		= &ADF4157_Data[6].chData;
volatile char*			adf4157RefDbl		= &ADF4157_Data[7].chData;
volatile int*		   	adf4157RCnt			= &ADF4157_Data[8].iData;

// R3
volatile char*		   	adf4157SigDelRst	= &ADF4157_Data[9].chData;
volatile char*		   	adf4157Ldp			= &ADF4157_Data[10].chData;
volatile char*			adf4157PdPol		= &ADF4157_Data[11].chData;
volatile char*			adf4157Pd			= &ADF4157_Data[12].chData;
volatile char*			adf4157CpZ			= &ADF4157_Data[13].chData;
volatile char*			adf4157CntRst		= &ADF4157_Data[14].chData;

// R4
volatile char*			adf4157NegBleed		= &ADF4157_Data[15].chData;
volatile char*			adf4157ClkDivMode	= &ADF4157_Data[16].chData;
volatile int*			adf4157ClkDivVal	= &ADF4157_Data[17].iData;

// SPI Error
volatile char* 			adf4157SPIErr		= &ADF4157_Data[18].chData;

// Modify buttons
volatile char* 		adf4157ModR0			= &ADF4157_Data[19].chData;
volatile char* 		adf4157ModR1			= &ADF4157_Data[20].chData;
volatile char* 		adf4157ModR2			= &ADF4157_Data[21].chData;
volatile char* 		adf4157ModR3			= &ADF4157_Data[22].chData;
volatile char* 		adf4157ModR4			= &ADF4157_Data[23].chData;
volatile char* 		adf4157ModAll			= &ADF4157_Data[24].chData;

int ADF4157_Init(void)
{
	int cfgValue = 0;

	// Setup SPI Interface
	SPI_Init(XPAR_SPI_0_BASEADDR, 0, 0, 0);

	// Setup Control GPIO Pins
	GPIO_SET_OUTPUT_PINS(XPAR_AXI_GPIO_0_BASEADDR, ~(GPIO_LE | GPIO_CE));
	GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_CE);
	GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LE);

	// R4
	cfgValue = ADF4157_BLEED(1) 		|
			   ADF4157_CLK_DIV_MODE(0) 	|
			   ADF4157_CLK_DIV_VAL(1) 	|
			   ADF4157_R4_CTRL;
	ADF4157_Set(cfgValue);

	// R3
	cfgValue = ADF4157_SIG_DEL_RST(0) 	|
			   ADF4157_LDP(0) 			|
			   ADF4157_PD_POL(1) 		|
			   ADF4157_PD(0) 			|
			   ADF4157_CP_Z(0) 			|
			   ADF4157_CNT_RST(0) 		|
			   ADF4157_R3_CTRL;
	ADF4157_Set(cfgValue);

	// R2
	cfgValue = ADF4157_CSR_EN(1)		|
			   ADF4157_CURR_SET(0)		|
			   ADF4157_PRESCALER(0)		|
			   ADF4157_RDIV2(1)			|
			   ADF4157_REF_DBL(0)		|
			   ADF4157_R_CNT(0)			|
			   ADF4157_R2_CTRL;
	ADF4157_Set(cfgValue);

	// R1
	cfgValue = ADF4157_FRAC_LSB_VAL(0)  |
			   ADF4157_R1_CTRL;
	ADF4157_Set(cfgValue);

	// R0
	cfgValue = ADF4157_MUXOUT_SDO		|
			   ADF4157_INT_VAL(0)		|
			   ADF4157_FRAC_MSB_VAL(0)	|
			   ADF4157_R0_CTRL;
	ADF4157_Set(cfgValue);

	return(1);
}

int ADF4157_Set(volatile int value)
{
	int validation = 0;
	txBuffer[0] = (unsigned char)((value & 0xFF000000) >> 24);
	txBuffer[1] = (unsigned char)((value & 0x00FF0000) >> 16);
	txBuffer[2] = (unsigned char)((value & 0x0000FF00) >> 8);
	txBuffer[3] = (unsigned char)(value & 0x000000FF);
	GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LE);
    SPI_TransferData(XPAR_AXI_SPI_0_BASEADDR,4,txBuffer,0,rxBuffer,1);
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LE);
    GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LE);
    SPI_TransferData(XPAR_AXI_SPI_0_BASEADDR,0,txBuffer,4,rxBuffer,1);
    validation += (int) ( rxBuffer[0] << 24 );
    validation += (int) ( rxBuffer[1] << 16 );
	validation += (int) ( rxBuffer[2] << 8 );
	validation += (int)rxBuffer[3];
	if (validation != value)
	{
		*adf4157SPIErr = 1;
		return(0);
	}
	else
	{
		*adf4157SPIErr = 0;
	}
    return(1);
}

int ADF4157_UpdateParameters(void)
{
	if(*adf4157ModAll != adf4157ModAllPrev)
	{
		ADF4157_SetR4();
		ADF4157_SetR3();
		ADF4157_SetR2();
		ADF4157_SetR1();
		ADF4157_SetR0();
		adf4157ModAllPrev = *adf4157ModAll;
	}
	/*
	if(*adf4157ModR0 != adf4157ModR0Prev)
	{
		ADF4157_SetR0();
		adf4157ModR0Prev = *adf4157ModR0;
	}

	if(*adf4157ModR1 != adf4157ModR1Prev)
	{
		ADF4157_SetR1();
		adf4157ModR1Prev = *adf4157ModR1;
	}

	if(*adf4157ModR2 != adf4157ModR2Prev)
	{
		ADF4157_SetR2();
		adf4157ModR2Prev = *adf4157ModR2;
	}

	if(*adf4157ModR3 != adf4157ModR3Prev)
	{
		adf4157ModR3Prev = *adf4157ModR3;
		ADF4157_SetR3();
	}

	if(*adf4157ModR4 != adf4157ModR4Prev)
	{
		adf4157ModR4Prev = *adf4157ModR4;
		ADF4157_SetR4();
	}
*/
	return(1);
}

int ADF4157_SetR0(void)
{
	int cfgValue = 0;

	// R0
	switch(*adf4157MuxOut)
	{
		case 0x00:
			cfgValue |= ADF4157_MUXOUT_Z;
			break;
		case 0x01:
			cfgValue |= ADF4157_MUXOUT_DVdd;
			break;
		case 0x02:
			cfgValue |= ADF4157_MUXOUT_DGND;
			break;
		case 0x03:
			cfgValue |= ADF4157_MUXOUT_RDIV;
			break;
		case 0x04:
			cfgValue |= ADF4157_MUXOUT_NDIV;
			break;
		case 0x05:
			cfgValue |= ADF4157_MUXOUT_ALOCK;
			break;
		case 0x06:
			cfgValue |= ADF4157_MUXOUT_DLOCK;
			break;
		case 0x07:
			cfgValue |= ADF4157_MUXOUT_SDO;
			break;
		case 0x08:
			cfgValue |= ADF4157_MUXOUT_CLKDIV;
			break;
		case 0x09:
			cfgValue |= ADF4157_MUXOUT_FLS;
			break;
		case 0x0A:
			cfgValue |= ADF4157_MUXOUT_RDIV2;
			break;
		case 0x0B:
			cfgValue |= ADF4157_MUXOUT_NDIV2;
			break;
		default:
			break;
	}
	cfgValue |= ADF4157_INT_VAL(*adf4157IntVal);
	cfgValue |= ADF4157_FRAC_MSB_VAL((*adf4157FracVal & 0x01FFE000) >> 13);
	cfgValue |= ADF4157_R0_CTRL;

	ADF4157_Set(cfgValue);

	return(1);
}

int ADF4157_SetR1(void)
{
	int cfgValue = 0;

	cfgValue |= ADF4157_FRAC_LSB_VAL(*adf4157FracVal & 0x00001FFF);
	cfgValue |= ADF4157_R1_CTRL;

	ADF4157_Set(cfgValue);

	return(1);
}

int ADF4157_SetR2(void)
{
	int cfgValue = 0;

	cfgValue |= ADF4157_CSR_EN(*adf4157CsrEn);
	cfgValue |= ADF4157_CURR_SET(*adf4157CurrSet);
	cfgValue |= ADF4157_PRESCALER(*adf4157Prescaler);
	cfgValue |= ADF4157_RDIV2(*adf4157RDiv2);
	cfgValue |= ADF4157_REF_DBL(*adf4157RefDbl);
	cfgValue |= ADF4157_R_CNT(*adf4157RCnt);
	cfgValue |= ADF4157_R2_CTRL;

	ADF4157_Set(cfgValue);

	return(1);
}

int ADF4157_SetR3(void)
{
	int cfgValue = 0;

	cfgValue |= ADF4157_SIG_DEL_RST(*adf4157SigDelRst);
	cfgValue |= ADF4157_LDP(*adf4157Ldp);
	cfgValue |= ADF4157_PD_POL(*adf4157PdPol);
	cfgValue |= ADF4157_PD(*adf4157Pd);
	cfgValue |= ADF4157_CP_Z(*adf4157CpZ);
	cfgValue |= ADF4157_CNT_RST(*adf4157CntRst);
	cfgValue |= ADF4157_R3_CTRL;

	ADF4157_Set(cfgValue);

	return(1);
}

int ADF4157_SetR4(void)
{
	int cfgValue = 0;

	cfgValue |= ADF4157_BLEED(*adf4157NegBleed);
	cfgValue |= ADF4157_CLK_DIV_MODE(0);
	cfgValue |= ADF4157_CLK_DIV_VAL(1);
	cfgValue |= ADF4157_R4_CTRL;

	ADF4157_Set(cfgValue);

	return(1);
}

